@@ -16,6 +16,8 @@
 #include "gtkgs.h"
 #include "prefs.h"
 
+#define GGV_GOAD_ID "bonobo-object:application-ps"
+
 static GdkCursor *pan_cursor = NULL;
 
 CORBA_Environment ev;
@@ -327,8 +329,10 @@ embeddable_factory (GnomeEmbeddableFactory *this, void *data)
 	/*
 	 * Interface GNOME::PersistStream 
 	 */
-	stream = gnome_persist_stream_new (load_ps_from_stream, NULL,
+	stream = gnome_persist_stream_new (GGV_GOAD_ID, 
+									   load_ps_from_stream, NULL,
 									   embeddable_data);
+	
 	if (stream == NULL){
 		gtk_object_unref (GTK_OBJECT (embeddable));
 		g_free (embeddable_data);
@@ -351,8 +355,8 @@ embeddable_factory (GnomeEmbeddableFactory *this, void *data)
 	/*
 	 * Bind the interfaces
 	 */
-	gtk_object_add_interface (GTK_OBJECT (embeddable),
-							  GTK_OBJECT (stream));
+	gnome_object_add_interface (GNOME_OBJECT (embeddable),
+								GNOME_OBJECT (stream));
 
 	gtk_signal_connect(GTK_OBJECT(embeddable), "destroy",
 					   GTK_SIGNAL_FUNC(embeddable_destroy_cb), embeddable_data);
@@ -395,7 +399,7 @@ main (int argc, char **argv)
 
 	load_prefs("/bonobo-application-ps/");
 
-	gtk_main ();
+	bonobo_main ();
 
 	CORBA_exception_free (&ev);
 
