@@ -250,6 +250,7 @@ gtk_gs_init (GtkGS *gs)
         gs->default_page_media = 1;
         gs->antialiased = TRUE;
         gs->override_media = FALSE;
+        gs->pages_marked = NULL;
 }
 
 static void
@@ -308,6 +309,12 @@ gtk_gs_cleanup (GtkGS *gs)
                 unlink(gs->gs_filename_unc);
                 gs->gs_filename_unc = NULL;
         }
+        if (gs->pages_marked) {
+                g_free(gs->pages_marked);
+                gs->pages_marked = NULL;
+        }
+                
+
         gs->doc = NULL;
   
 }
@@ -1149,7 +1156,7 @@ gboolean
 gtk_gs_load (GtkGS *gs, char *fname)
 {
         struct stat sbuf;
-	
+	int i;
 	g_assert(gs != NULL);
         /*
         printf (" in gs_load\n");
@@ -1229,6 +1236,10 @@ gtk_gs_load (GtkGS *gs, char *fname)
                 if ((!gs->doc->epsf && gs->doc->numpages>0) || 
                     (gs->doc->epsf && gs->doc->numpages>1)) {
                         gs->structured_doc = TRUE; 
+                        gs->pages_marked = g_malloc(sizeof(gint) * gs->doc->numpages
+);
+                        for (i=0;i<gs->doc->numpages; i++)
+                                gs->pages_marked[i] = 0;
                 }
                 else {
                         gs->structured_doc = FALSE; 
@@ -1629,3 +1640,19 @@ gtk_gs_document_page_label(GtkGS *widget, int page)
         
 	return NULL;
 }
+
+gint 
+gtk_gs_count_marked_pages(GtkGS *widget)
+{
+        int i;
+        int count=0;
+	g_return_val_if_fail(widget != NULL, 0);
+	g_return_val_if_fail(GTK_IS_GS(widget), 0);
+
+        if (!widget->structured_doc)
+                return 0;
+        for (i=0;i<widget->doc->numpages; i++)
+                if (widget->pages_marked[i])
+                        count++;
+        return count;
+}
