@@ -100,6 +100,7 @@ struct _GtkGS
 	gchar      *gs_uncompress_cmd;        /* set to gzip -cd */
         gfloat     xdpi, ydpi;
         gint       orientation;
+        gint       *pages_marked;
 };
 
 struct _GtkGSClass
@@ -147,7 +148,7 @@ gint gtk_gs_get_orientation(GtkGS *gs);
 gchar *gtk_gs_document_title(GtkGS *widget);
 guint gtk_gs_document_numpages(GtkGS *widget);
 gchar *gtk_gs_document_page_label(GtkGS *widget, int page);
-
+gint gtk_gs_count_marked_pages(GtkGS *widget);
 
 #ifdef __cplusplus
 }
