@@ -16,6 +16,8 @@
 #include "gtkgs.h"
 #include "crop.h"
 #include "prefs.h"
+#include "foot.xpm"
+#include "foot_white.xpm"
 
 /* This sizes are good for the worst case: laptops :) */
 
@@ -26,8 +28,18 @@ enum {
         GEOMETRY_KEY = -1,
         SPARTAN_KEY = -2,
         NO_MENU_BAR_KEY = -3,
+        FULL_SCREEN_KEY = -4,
 };
 
+enum {
+        TOGGLE_CURRENT_PAGE = -1,
+        TOGGLE_EVEN_PAGES = -2,
+        TOGGLE_ODD_PAGES = -3,
+        TOGGLE_ALL_PAGES = -4,
+        UNMARK_ALL_PAGES = -5,
+};
+
+
 static gint width = DEFAULT_WINDOW_WIDTH, height = DEFAULT_WINDOW_HEIGHT;
 
 enum {
@@ -54,6 +66,7 @@ struct _ggv_window {
         GtkWidget *popup_menu;
         GtkWidget *sidebar;
         GtkWidget *file_sel;
+        GtkWidget *save_file_sel;
         GtkWidget *appbar;
         GtkObject *hadj, *vadj;
         GtkWidget *gs;
@@ -69,6 +82,7 @@ struct _ggv_window {
         /* store some pointers to widgets from GnomeUIInfo trees here
            as we need them for setting sensitivity and stuff like that */
         GtkWidget *nextbutton, *nextitem, *nextpu;  /* next button, menu item and popup menu item */
+        GtkWidget *save, *saveas;
         GtkWidget *prevbutton, *previtem, *prevpu;  /* prev ... */
         GtkWidget *printitem, *reloaditem;          /* menu items */
         GtkWidget *zoominbutton;                    /* zoom buttons */
@@ -92,6 +106,11 @@ struct _ggv_prefs
 GList *window_list;
 static int active_ggv;
 static ggv_prefs prefs_dialog = { NULL };
+static GdkPixmap *page_marker_pixmap;
+static GdkBitmap *page_marker_mask;
+
+static GdkPixmap *page_marker_white_pixmap;
+static GdkBitmap *page_marker_white_mask;
 
 /* prototypes */
 static gboolean load_gs(ggv_window *ggv, char *fname);
@@ -108,8 +127,10 @@ static void goto_page(ggv_window *ggv, int page);
 static void set_page_sensitivities(ggv_window *ggv, int page);
 static void flash_message(ggv_window *ggv, gchar *flash);
 static void error_message(ggv_window *ggv, gchar *errormsg);
+static void toggle_pages(ggv_window *ggv, gint what_to_toggle, gint page_number);
 
 /* callback prototypes */
+
 static void about_callback(GtkWidget *widget, gpointer data);
 static void new_callback(GtkWidget *widget, gpointer data);
 static void button_press_callback(GtkWidget *widget, GdkEventButton *event, gpointer data);
@@ -151,6 +172,7 @@ static void recent_cb(GtkWidget *w, gpointer *data);
 static char *geometry = 0;
 static gboolean spartan = FALSE;
 static gint     nomenubar = FALSE;
+static gboolean fullscreen=FALSE;
 
 static int
 get_number (const char **geometry)
@@ -256,7 +278,7 @@ recent_cb(GtkWidget *w, gpointer *data)
                         wwd->ggv->loaded = load_gs(wwd->ggv, (char*)wwd->data);
                         return;
                 }
-                ErrorMessage = g_strdup_printf(_("%s is not a file."), (char*)wwd->data);
+                ErrorMessage = g_strdup_printf(_("%s is not a regular file."), (char*)wwd->data);
 
         }
         else {
@@ -268,6 +290,25 @@ recent_cb(GtkWidget *w, gpointer *data)
         /*        g_free((char*)wwd->data); */
 }
 
+static char *
+escape_underlines (char *str)
+{
+	char *buf;
+	char *p;
+
+	buf = g_new (char, 2 * strlen (str) + 1);
+	
+	for (p = buf; *str; str++) {
+		if (*str == '_')
+			*p++ = '_';
+
+		*p++ = *str;
+	}
+
+	*p = '\0';
+
+	return buf;
+}
 
 static void
 recent_update_menus (ggv_window *ggv, GList *recent_files)
@@ -277,8 +318,8 @@ recent_update_menus (ggv_window *ggv, GList *recent_files)
 	gchar *path;
 	int i;
         window_with_data *wwd;
-
-
+        char *escape;
+        
         if (!ggv->show_menus)
                 return;
         /*
@@ -316,7 +357,11 @@ recent_update_menus (ggv_window *ggv, GList *recent_files)
 		/*data->window = window;*/
 		menu->label = g_new (gchar, 
 			strlen (g_list_nth_data (recent_files, i)) + 5);
-		sprintf (menu->label, "_%i. %s", i+1, (gchar*)g_list_nth_data (recent_files, i));
+
+                escape = escape_underlines ((gchar*)g_list_nth_data (recent_files, i));
+		sprintf (menu->label, "_%i. %s", i+1, escape);
+                g_free (escape);
+                
 		menu->type = GNOME_APP_UI_ITEM;
 		menu->hint = NULL;
                 
@@ -429,6 +474,8 @@ load_gs(ggv_window *ggv, char *fname)
                 gtk_widget_set_sensitive (ggv->zoomoutbutton, FALSE);
                 gtk_widget_set_sensitive (ggv->zoominbutton, FALSE);
                 gtk_widget_set_sensitive (ggv->printitem, FALSE);
+                gtk_widget_set_sensitive (ggv->save, FALSE);
+                gtk_widget_set_sensitive (ggv->saveas, FALSE);
                 flash = g_strdup_printf(_("Unable to load %s."), fname);
                 error_message(ggv, flash);
                 return FALSE;
@@ -488,6 +535,8 @@ load_gs(ggv_window *ggv, char *fname)
         gtk_widget_set_sensitive (ggv->zoominbutton, TRUE);
         gtk_widget_set_sensitive (ggv->printitem, TRUE);
         gtk_widget_set_sensitive (ggv->reloaditem, TRUE);
+        gtk_widget_set_sensitive (ggv->saveas, TRUE);
+        gtk_widget_set_sensitive (ggv->save, TRUE);
 
         flash = g_strdup_printf(_("Successfully loaded %s."), fname);
         flash_message(ggv, flash);
@@ -496,9 +545,54 @@ load_gs(ggv_window *ggv, char *fname)
 }
 
 
+
   /****************/
  /* Callbacks... */
 /****************/
+
+static void unmark_all_pages_callback(GtkWidget *widget, gpointer data) 
+{
+        ggv_window *ggv = (ggv_window *) data;
+        g_return_if_fail (ggv!=NULL);
+
+        toggle_pages(ggv, UNMARK_ALL_PAGES, GTK_GS(ggv->gs)->current_page);
+}
+
+
+static void toggle_current_page_callback(GtkWidget *widget, gpointer data) 
+{
+        ggv_window *ggv = (ggv_window *) data;
+        g_return_if_fail (ggv!=NULL);
+
+        toggle_pages(ggv, TOGGLE_CURRENT_PAGE, GTK_GS(ggv->gs)->current_page);
+}
+
+
+static void toggle_even_pages_callback(GtkWidget *widget, gpointer data) 
+{
+        ggv_window *ggv = (ggv_window *) data;
+        g_return_if_fail (ggv!=NULL);
+
+        toggle_pages(ggv, TOGGLE_EVEN_PAGES, 0);
+}
+
+static void toggle_odd_pages_callback(GtkWidget *widget, gpointer data) 
+{
+        ggv_window *ggv = (ggv_window *) data;
+        g_return_if_fail (ggv!=NULL);
+
+        toggle_pages(ggv, TOGGLE_ODD_PAGES, 0);
+}
+
+static void toggle_all_pages_callback(GtkWidget *widget, gpointer data) 
+{
+        ggv_window *ggv = (ggv_window *) data;
+        g_return_if_fail (ggv!=NULL);
+
+        toggle_pages(ggv, TOGGLE_ALL_PAGES, 0);
+}
+
+
 static void about_callback(GtkWidget *widget, gpointer data) 
 {
 	GtkWidget *about = NULL;
@@ -514,7 +608,7 @@ static void about_callback(GtkWidget *widget, gpointer data)
 				"Copyright (C) 1998, 1999 the Free Software Foundation",
 				(const gchar **) authors,
 				_("Postscript(TM) document viewer.\nBased on Tim Theisen's excellent Ghostview application."),
-				NULL);
+				"ggv-splash.png");
         gtk_widget_show(about);
 }
 
@@ -576,6 +670,8 @@ static void button_press_callback(GtkWidget *widget, GdkEventButton *event, gpoi
         default:
                 break;
         }
+        gtk_clist_select_row (GTK_CLIST (ggv->pagelist),  GTK_GS(ggv->gs)->current_page,1);
+
 }
 
 static void
@@ -957,6 +1053,10 @@ static void open_callback(GtkWidget *widget, gpointer data)
         gtk_widget_show(ggv->file_sel);
 }
 
+
+
+
+
 static void orientation_callback(GtkWidget *widget, gpointer data)
 {
         window_with_data *wwd = (window_with_data *) data;
@@ -1014,10 +1114,136 @@ static void reload_callback(GtkWidget *widget, gpointer data)
                 g_free (fname);
         }
 }
+static void save_file_open_cancel_callback(GtkWidget *widget, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *)data;
 
-static void save_callback(GtkWidget *widget, gpointer data) {}
+        gtk_widget_destroy(ggv->save_file_sel);
+        ggv->save_file_sel = NULL;
+}
+
+
+static void save_file_open_delete_callback(GtkWidget *widget, GdkEventAny *e, gpointer data)
+{
+        save_file_open_cancel_callback(widget, data);
+}
+
+
+static void save_file_open_ok_callback(GtkWidget *widget, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *)data;
+        gchar *file;
+        gchar *baseFileName, *flash;
+        char *ErrorMessage;
+        struct stat StatRecord;
+        int i;
+        FILE *FileHandler;
+
+	file = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(ggv->save_file_sel)));
+
+        if (stat(file,&StatRecord) == 0) {
+                
+                if (S_ISDIR(StatRecord.st_mode)) {
+                        /* This is a directory, reset window and
+                           try again */
+                        if (strlen(file) > 0 && file[strlen(file)-1] != '/')
+                                strcat(file,"/");
+                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(ggv->save_file_sel),
+                                                        file);
+                        return;
+                }
+                
+        }
+        else {
+                /* We have to check if there is a wildcard in the directory name
+                 */
+                
+                baseFileName = g_basename(file);
+                if (strchr(baseFileName, '?' ) != NULL ||
+                    strchr(baseFileName, '*' ) != NULL) {
+                        gtk_file_selection_complete(GTK_FILE_SELECTION(ggv->save_file_sel),
+                                                    file);
+                        return;
+                }
+                /* At this point, we know the file does not exist */
+
+        }
+        gtk_widget_destroy(ggv->save_file_sel);
+        ggv->save_file_sel = NULL;
+
+        /* Do your thing */
+
+        if (gtk_gs_count_marked_pages(GTK_GS (ggv->gs)) == 0) {
+                ErrorMessage = g_strdup(_("There are no marked pages."));
+                error_message(ggv,ErrorMessage);
+                return;
+        }
+        
+        g_print("Pages marked %d\n",
+                gtk_gs_count_marked_pages(GTK_GS (ggv->gs)));
+
+        if ((FileHandler = fopen(file, "w")) != NULL) {
+
+                pscopydoc(FileHandler, GTK_GS (ggv->gs)->gs_filename,
+                          GTK_GS (ggv->gs)->doc, 
+                          GTK_GS (ggv->gs)->pages_marked);
+
+                flash = g_strdup_printf(_("Successfully saved maked pages to file %s."), file);
+                flash_message(ggv, flash);
+        }
+        else {
+                ErrorMessage = g_strdup_printf(_("Unable to create %s."),file);
+                error_message(ggv,ErrorMessage);
+        }
+
+        g_free(file);
+}
+
+
+static void save_callback(GtkWidget *widget, gpointer data) {
+        ggv_window *ggv = (ggv_window *) data;
+        char *ErrorMessage;
+        gchar *dirname;
+
+        if (ggv->loaded) {
+
+                if(ggv->save_file_sel)
+                        return;
+                
+                ggv->save_file_sel = gtk_file_selection_new(_("ggv: Save marked pages"));
+                gtk_signal_connect(GTK_OBJECT(ggv->save_file_sel), "delete_event",
+                                   GTK_SIGNAL_FUNC(save_file_open_delete_callback), data);
+                gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(ggv->save_file_sel)->cancel_button),
+                                   "clicked",
+                                   GTK_SIGNAL_FUNC(save_file_open_cancel_callback), data);
+                gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(ggv->save_file_sel)->ok_button),
+                                   "clicked",
+                                   GTK_SIGNAL_FUNC(save_file_open_ok_callback), data);
+                
+                gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(ggv->save_file_sel));
+                
+                /* Switch to the directory where the current 
+                   file came from */
+                
+                if (GTK_GS (ggv->gs)->gs_filename != NULL &&
+                    (dirname=g_dirname(GTK_GS (ggv->gs)->gs_filename)) != NULL)
+                        gtk_file_selection_complete(GTK_FILE_SELECTION(ggv->save_file_sel),
+                                                    dirname);
+                
+                gtk_widget_show(ggv->save_file_sel);
+                
+        }
+}
+
+static void save_as_callback(GtkWidget *widget, gpointer data) 
+{
+        ggv_window *ggv = (ggv_window *) data;
+        if (ggv->loaded) {
+
+        }
+
+}
 
-static void save_as_callback(GtkWidget *widget, gpointer data) {}
 
 static void
 scrollpane_middle_click_callback(GtkWidget *widget, gpointer data)
@@ -1039,15 +1265,126 @@ scrollpane_right_click_callback(GtkWidget *widget, gpointer data)
         }
 }
 
+/* Convenience function to load a pixmap and mask from xpm data */
+static void
+create_pixmap (char **data, GdkPixmap **pixmap, GdkBitmap **mask)
+{
+	GdkImlibImage *im;
+
+	im = gdk_imlib_create_image_from_xpm_data (data);
+	gdk_imlib_render (im, im->rgb_width, im->rgb_height);
+	*pixmap = gdk_imlib_copy_image (im);
+	*mask = gdk_imlib_copy_mask (im);
+	gdk_imlib_destroy_image (im);
+}
+
+static void toggle_pages(ggv_window *ggv, gint what_to_toggle, gint page_number)
+{
+        gint n;
+
+
+        gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
+
+        /*
+          gtk_clist_get_text(GTK_CLIST(ggv->pagelist), row, 0, &CurrentMarker);
+          
+          if (strcmp(CurrentMarker,"x")==0)
+          NewMarker = "";
+          else
+          NewMarker = "x";
+        */
+        switch (what_to_toggle) {
+        case TOGGLE_CURRENT_PAGE:
+                if (GTK_GS(ggv->gs)->pages_marked[page_number] = !GTK_GS(ggv->gs)->pages_marked[page_number])
+                        gtk_clist_set_pixmap(GTK_CLIST(ggv->pagelist), page_number, 0, 
+                                             page_marker_pixmap, page_marker_mask);
+                else
+                        gtk_clist_set_pixmap(GTK_CLIST(ggv->pagelist), page_number, 0, 
+                                             page_marker_white_pixmap, page_marker_white_mask);
+                break;
+        case TOGGLE_ALL_PAGES:
+        case TOGGLE_EVEN_PAGES:
+        case TOGGLE_ODD_PAGES:
+                n = gtk_gs_document_numpages(GTK_GS (ggv->gs));
+                for (page_number=(what_to_toggle==TOGGLE_EVEN_PAGES)?1:0;
+                     page_number<n;
+                     page_number+=(what_to_toggle==TOGGLE_ALL_PAGES)?1:2)
+                        if (GTK_GS(ggv->gs)->pages_marked[page_number] = !GTK_GS(ggv->gs)->pages_marked[page_number])
+                                gtk_clist_set_pixmap(GTK_CLIST(ggv->pagelist), page_number, 0, 
+                                                     page_marker_pixmap, page_marker_mask);
+                        else
+                                gtk_clist_set_pixmap(GTK_CLIST(ggv->pagelist), page_number, 0, 
+                                                     page_marker_white_pixmap, page_marker_white_mask);
+                break;
+        case UNMARK_ALL_PAGES:
+                n = gtk_gs_document_numpages(GTK_GS (ggv->gs));
+                for (page_number=0; page_number<n; page_number++) {
+                        GTK_GS(ggv->gs)->pages_marked[page_number]  = FALSE;
+                        gtk_clist_set_pixmap(GTK_CLIST(ggv->pagelist), page_number, 0, 
+                                             page_marker_white_pixmap, page_marker_white_mask);
+                        
+                }
+                break;
+        }
+
+        /*
+          gtk_clist_set_text(GTK_CLIST(ggv->pagelist),
+          
+          row, 0, NewMarker);
+        */
+        gtk_clist_thaw(GTK_CLIST(ggv->pagelist));
+}
+
+
+
+static void
+select_page_button_press_callback(GtkWidget *widget, GdkEventButton *event, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *)data;
+        gint row = 0;
+        gint col = 0;
+        char *CurrentMarker;
+        char *NewMarker;
+        GtkWidget *marker;
+        
+        /*
+        marker = gnome_stock_pixmap_widget_new(retval->prevbutton, GNOME_STOCK_MENU_BACK);
+        marker = gnome_stock_pixmap_widget_new(retval->prevbutton, GNOME_STOCK_MENU_BACK);
+        */
+
+        marker = gnome_stock_new_with_icon(GNOME_STOCK_MENU_BACK);
+        g_print("Button %u\n", event->button); 
+
+        if (GTK_CLIST(ggv->pagelist)->clist_window == event->window) {
+                gtk_clist_get_selection_info (GTK_CLIST(widget), event->x, event->y, &row, &col);
+                g_print("Button %u: %d %d\n", event->button, row, col); 
+                if (event->button == 3 & row >= 0) {
+                        toggle_pages(ggv, TOGGLE_CURRENT_PAGE, row);
+                }
+        }
+
+}
+
+
 static void 
 select_page_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
 {
         ggv_window *ggv = (ggv_window *)data;
 
-        if(GTK_GS(ggv->gs)->current_page != row) {
-                gtk_gs_goto_page(GTK_GS(ggv->gs), row);
-                row = GTK_GS(ggv->gs)->current_page; /* make sure */
-                set_page_sensitivities(ggv, row);
+        g_return_if_fail(widget != NULL);
+
+
+        /* If event is null then it is not from the user */
+
+        if (event == NULL || event->button == 1) {
+                if(GTK_GS(ggv->gs)->current_page != row) {
+                        gtk_gs_goto_page(GTK_GS(ggv->gs), row);
+                        row = GTK_GS(ggv->gs)->current_page; /* make sure */
+                        set_page_sensitivities(ggv, row);
+                }
+        }
+        if (event != NULL && event->button == 3) {
+                g_print("Button %u\n", event->button); 
         }
 }
 
@@ -1185,6 +1522,9 @@ create_popup_menus(ggv_window *ggv)
                 {GNOME_APP_UI_ITEM, N_("Recenter Page"), NULL, recenter_page_callback, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_SEPARATOR},
+                {GNOME_APP_UI_ITEM, N_("Mark even pages"), NULL, toggle_even_pages_callback, NULL, NULL,
+                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+                {GNOME_APP_UI_SEPARATOR},
                 {GNOME_APP_UI_TOGGLEITEM, N_("Show Menus"), NULL, show_menubar_callback, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_TOGGLEITEM, N_("Show Side Panel"), NULL, hide_panel_callback, NULL, NULL,
@@ -1250,16 +1590,16 @@ create_menus(ggv_window *retval, window_with_data *temp)
 
         GnomeUIInfo fileMenu[] = {
                 GNOMEUIINFO_MENU_NEW_ITEM(N_("_New Window"), N_("Open a new window"), new_callback, NULL),
-                {GNOME_APP_UI_ITEM, N_("_Open"), N_("Load a postscript document"), open_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Open..."), N_("Load a postscript document"), open_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 'o', GDK_CONTROL_MASK, NULL},
                 {GNOME_APP_UI_ITEM, N_("_Reload"), N_("Reload current document from disk"), reload_callback, retval, NULL,
-                 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REVERT, 'r', GDK_CONTROL_MASK, NULL},
+                 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH, 'r', GDK_CONTROL_MASK, NULL},
                 {GNOME_APP_UI_SEPARATOR},
-                {GNOME_APP_UI_ITEM, N_("_Print..."), N_("Print the document"), print_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Print"), N_("Print the document"), print_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PRINT, 'p', GDK_CONTROL_MASK, NULL },
-                {GNOME_APP_UI_ITEM, N_("_Save"), NULL, save_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("Save _marked pages..."), NULL, save_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, N_("S_ave as..."), NULL, save_as_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("Save as..."), NULL, save_as_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, NULL},
                 {GNOME_APP_UI_SEPARATOR},
                 {GNOME_APP_UI_ITEM, N_("_Close"), N_("Close this window"), close_callback, retval, NULL,
@@ -1279,6 +1619,17 @@ create_menus(ggv_window *retval, window_with_data *temp)
                 {GNOME_APP_UI_ITEM, N_("_Recenter Page"), N_("Center page"), recenter_page_callback, retval, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_SEPARATOR},
+                {GNOME_APP_UI_ITEM, N_("Toggle _current page"), NULL, toggle_current_page_callback, retval,NULL,
+                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+                {GNOME_APP_UI_ITEM, N_("Toggle _all pages"), NULL, toggle_all_pages_callback, retval,NULL,
+                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+                {GNOME_APP_UI_ITEM, N_("Toggle _even pages"), NULL, toggle_even_pages_callback, retval,NULL,
+                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+                {GNOME_APP_UI_ITEM, N_("Toggle _odd pages"), NULL, toggle_odd_pages_callback, retval,NULL,
+                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+                {GNOME_APP_UI_ITEM, N_("_Unmark all pages"), NULL, unmark_all_pages_callback, retval,NULL,
+                 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+                {GNOME_APP_UI_SEPARATOR},
                 {GNOME_APP_UI_SUBTREE, N_("_Orientation"), NULL, orientationMenu, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_SUBTREE, N_("_Zoom"), NULL, zoomMenu, NULL, NULL,
@@ -1310,8 +1661,11 @@ create_menus(ggv_window *retval, window_with_data *temp)
         retval->previtem = documentMenu[1].widget;
         retval->reloaditem = fileMenu[2].widget;
         retval->printitem = fileMenu[4].widget;
+        retval->save = fileMenu[5].widget;
+        retval->saveas = fileMenu[6].widget;
         retval->menus_vis = settingsMenu[2].widget;
         retval->panel_vis = settingsMenu[3].widget;
+
         gtk_widget_set_sensitive (GTK_WIDGET(retval->previtem), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(retval->nextitem), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(retval->printitem), FALSE);
@@ -1389,12 +1743,19 @@ create_sidebar(ggv_window *retval)
 
         retval->pagelist = gtk_clist_new(2);
 	gtk_widget_set_usize (retval->pagelist, 80, 100);
-	gtk_clist_set_column_width (GTK_CLIST (retval->pagelist), 1, 10);
-	gtk_clist_set_column_width (GTK_CLIST (retval->pagelist), 2, 20);
+
+	gtk_clist_set_column_width (GTK_CLIST (retval->pagelist), 0, 18);
+	gtk_clist_set_column_width (GTK_CLIST (retval->pagelist), 1, 20);
 	gtk_clist_set_selection_mode (GTK_CLIST (retval->pagelist), GTK_SELECTION_BROWSE);
-	gtk_clist_set_column_justification (GTK_CLIST (retval->pagelist), 1, GTK_JUSTIFY_CENTER);
-	gtk_clist_set_column_justification (GTK_CLIST (retval->pagelist), 2, GTK_JUSTIFY_RIGHT);
+	gtk_clist_set_column_justification (GTK_CLIST (retval->pagelist), 0, GTK_JUSTIFY_LEFT);
+	gtk_clist_set_column_justification (GTK_CLIST (retval->pagelist), 1, GTK_JUSTIFY_RIGHT);
+
+        gtk_clist_set_shadow_type (GTK_CLIST (retval->pagelist),GTK_SHADOW_ETCHED_IN);
+
         gtk_signal_connect(GTK_OBJECT(retval->pagelist), "select_row", GTK_SIGNAL_FUNC(select_page_callback),retval);
+
+        gtk_signal_connect(GTK_OBJECT(retval->pagelist), "button_press_event", GTK_SIGNAL_FUNC(select_page_button_press_callback),retval);
+
 	gtk_container_add (GTK_CONTAINER (retval->pagelistscroll), retval->pagelist);
 	gtk_widget_show (retval->pagelist);
 	gtk_widget_show (retval->pagelistscroll);
@@ -1447,7 +1808,7 @@ ggv_window *create_ggv_window()
                 int xpos, ypos;
          
                 ggv_parse_geometry (geometry, &xpos, &ypos, &bxneg, &byneg, &width, &height);
-                g_print("Geometry %u %u %u %u\n", xpos, ypos, width, height);
+                g_print("Geometry %d %d %d %d\n", xpos, ypos, width, height);
                 
                 if (xpos != -1 && ypos != -1) {
                         if (bxneg) xpos -= width;
@@ -1463,7 +1824,14 @@ ggv_window *create_ggv_window()
          /* Only the first window gets --geometry treatment for now */
                 geometry = NULL;
         }
+        if (fullscreen) {
+                gtk_widget_set_uposition (GTK_WIDGET (retval->main_window), 0, 0);
+                /* FIXME we need to know the vertical size of
+                   the current window bar */
+                height = gdk_screen_height () - 18;
+                width = gdk_screen_width ();
 
+        }
         gtk_window_set_default_size(GTK_WINDOW(retval->main_window), 
                                     width, height);
         gtk_window_set_policy(GTK_WINDOW(retval->main_window), TRUE, TRUE, FALSE);
@@ -1560,6 +1928,7 @@ ggv_window *create_ggv_window()
         gtk_signal_connect (GTK_OBJECT(retval->main_window), "key_press_event", GTK_SIGNAL_FUNC(key_pressed_event_callback), retval);
 
         retval->file_sel = NULL;
+        retval->save_file_sel = NULL;
 
 
 
@@ -1749,6 +2118,7 @@ open_prefs_dialog(ggv_prefs *pd)
         gtk_widget_show(label);
         pd->gs = gtk_entry_new();
         gtk_entry_set_text(GTK_ENTRY(pd->gs), gs_cmd);
+	gtk_entry_set_position(GTK_ENTRY(pd->gs), 0);
         gtk_table_attach(GTK_TABLE(table), pd->gs,
                          1, 2, 0, 1,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -1767,6 +2137,7 @@ open_prefs_dialog(ggv_prefs *pd)
         gtk_widget_show(label);
         pd->scan_pdf = gtk_entry_new();
         gtk_entry_set_text(GTK_ENTRY(pd->scan_pdf), gs_scan_pdf_cmd);
+	gtk_entry_set_position(GTK_ENTRY(pd->scan_pdf), 0);
         gtk_table_attach(GTK_TABLE(table), pd->scan_pdf,
                          1, 2, 1, 2,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -1785,6 +2156,7 @@ open_prefs_dialog(ggv_prefs *pd)
         gtk_widget_show(label);
         pd->uncompress = gtk_entry_new();
         gtk_entry_set_text(GTK_ENTRY(pd->uncompress), gs_uncompress_cmd);
+	gtk_entry_set_position(GTK_ENTRY(pd->uncompress), 0);
         gtk_table_attach(GTK_TABLE(table), pd->uncompress,
                          1, 2, 2, 3,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -1812,6 +2184,7 @@ open_prefs_dialog(ggv_prefs *pd)
         gtk_widget_show(label);
         pd->print = gtk_entry_new();
         gtk_entry_set_text(GTK_ENTRY(pd->print), gs_print_cmd);
+	gtk_entry_set_position(GTK_ENTRY(pd->print), 0);
         gtk_table_attach(GTK_TABLE(table), pd->print,
                          1, 2, 0, 1,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -1875,6 +2248,10 @@ parse_an_arg (poptContext state,
      case NO_MENU_BAR_KEY:
              nomenubar = TRUE;
              break;
+     case FULL_SCREEN_KEY:
+             fullscreen = TRUE;
+             break;
+
      }
 
 }
@@ -1892,6 +2269,9 @@ static struct poptOption options_callback [] = {
 	  N_("Do not use a menu bar"), N_("NOMENUBAR") },
 
 
+	{ "fullscreen", '\0', POPT_ARG_NONE, NULL, FULL_SCREEN_KEY,
+	  N_("Use the whole screen for the main window"), N_("FULLSCREEN") },
+
         
         { NULL, '\0', 0, NULL, 0}
 };
@@ -1933,6 +2313,10 @@ main(int argc, char *argv[])
 
         load_prefs("/ggv/");
 
+	create_pixmap (foot_xpm, &page_marker_pixmap, &page_marker_mask);
+	create_pixmap (foot_white_xpm, &page_marker_white_pixmap, &page_marker_white_mask);
+
+
         startups = poptGetArgs(ctx);
         if(startups) {
                 for(i = 0; startups[i]; i++) {
