@@ -18,9 +18,16 @@ extern gchar *gs_print_cmd;
 extern gint gs_def_media;
 extern gboolean gs_antialiased, gs_override_media;
 extern gboolean gs_panel, gs_menubar;
+extern gint gs_num_recent;
 
 void load_prefs(gchar *);
 void save_prefs(gchar *);
 void set_prefs (GtkGS *);
 
+#ifndef MAX_RECENT
+#define MAX_RECENT 6
+#endif
+
+
+
 #endif
