@@ -1528,16 +1528,20 @@ pscopyuntil(from, to, begin, end, comment)
     int i;
     char buf[BUFSIZ];
     char *cp;
+    int temp;
 
-    comment_length = strlen(comment);
+    if (comment != NULL)
+      comment_length = strlen(comment);
+    else
+      comment_length = 0;
     if (begin >= 0) fseek(from, begin, SEEK_SET);
-    while (ftell(from) < end) {
 
+    while (ftell(from) < end) {
 	fgets(line, sizeof line, from);
 
 	/* iscomment cannot be used here,
 	 * because comment_length is not known at compile time. */
-	if (strncmp(line, comment, comment_length) == 0) {
+	if (comment != NULL && strncmp(line, comment, comment_length) == 0) {
 	    cp = (char *) g_malloc(strlen(line)+1);
 	    if (cp == NULL) {
 		fprintf(stderr, "Fatal Error: Dynamic memory exhausted.\n");
@@ -1587,12 +1591,117 @@ pscopyuntil(from, to, begin, end, comment)
  *	blank -- determine whether the line contains nothing but whitespace.
  */
 
-static int
-blank(line)
-    char *line;
+static int blank(char *line)
 {
-    char *cp = line;
-
-    while (*cp == ' ' || *cp == '\t') cp++;
-    return *cp == '\n' || (*cp == '%' && (line[0] != '%' || line[1] != '%'));
+  char *cp = line;
+  
+  while (*cp == ' ' || *cp == '\t') cp++;
+  return *cp == '\n' || (*cp == '%' && (line[0] != '%' || line[1] != '%'));
 }
+
+/*##########################################################*/
+/* pscopydoc */
+/* Copy the headers, marked pages, and trailer to fp */
+/*##########################################################*/
+
+void pscopydoc(FILE *dest_file,
+	       char *src_filename,
+	       struct document *d,
+	       gint *pagelist)
+{
+  FILE *src_file;
+  char text[PSLINELENGTH];
+  char *comment;
+  gboolean pages_written = FALSE;
+  gboolean pages_atend = FALSE;
+  int pages;
+  int page = 1;
+  int i, j;
+  int here;
+  /*
+  FileData fd;
+  */
+  
+  src_file = fopen(src_filename, "r");
+  /*
+  fd = ps_io_init(src_file);
+  */
+  i=0;
+  pages=0;
+  for (i=0;i<d->numpages;i++)    {
+    if (pagelist[i]) pages++;
+  }
+
+  g_print("number of pages to be copied %d out of [%d]\n",pages, d->numpages);
+
+  here = d->beginheader;
+
+  while ((comment=pscopyuntil(src_file,dest_file,here,d->endheader,"%%Pages:"))) {
+
+       here = ftell(src_file);
+       if (pages_written || pages_atend) {
+          g_free(comment);
+          continue;
+       }
+       sscanf(comment+length("%%Pages:"), "%s", text);
+       if (strcmp(text, "(atend)") == 0) {
+          fputs(comment, dest_file);
+          pages_atend = TRUE;
+       } else {
+          switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+             case 1:
+                fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
+                break;
+             default:
+                fprintf(dest_file, "%%%%Pages: %d\n", pages);
+                break;
+          }
+          pages_written = TRUE;
+       }
+       g_free(comment);
+  }
+   pscopyuntil(src_file, dest_file, d->beginpreview, d->endpreview,NULL);
+   pscopyuntil(src_file, dest_file, d->begindefaults, d->enddefaults,NULL);
+   pscopyuntil(src_file, dest_file, d->beginprolog, d->endprolog,NULL);
+   pscopyuntil(src_file, dest_file, d->beginsetup, d->endsetup,NULL);
+
+   for (i = 0; i < d->numpages; i++) {
+     if (d->pageorder == DESCEND) j = (d->numpages - 1) - i;
+     else                         j = i;
+
+     j = i;
+     if (pagelist[j]) {
+
+       comment = pscopyuntil(src_file,dest_file,
+			     d->pages[i].begin,d->pages[i].end, "%%Page:");
+       fprintf(dest_file, "%%%%Page: %s %d\n",d->pages[i].label, page++);
+       g_free(comment);
+       pscopyuntil(src_file, dest_file, -1, d->pages[i].end,NULL);
+     }
+   }
+
+   here = d->begintrailer;
+   while ((comment = pscopyuntil(src_file, dest_file, here, d->endtrailer, "%%Pages:"))) {
+      here = ftell(src_file);
+      if (pages_written) {
+         g_free(comment);
+         continue;
+      }
+      switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+         case 1:
+            fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
+            break;
+         default:
+            fprintf(dest_file, "%%%%Pages: %d\n", pages);
+            break;
+      }
+      pages_written = TRUE;
+      g_free(comment);
+   }
+
+   fclose(dest_file);
+   fclose(src_file);
+
+   g_print("pscopydoc end\n");
+}
+
